/*
 * This is NICEditor, by Tim Koop
 *
 * No warrenty whatsoever comes with it.
 * NICEditor is based on:
 * FCKeditor - The text editor for internet
 * Copyright (C) 2003-2004 Frederico Caldeira Knabben
 * 
 * Licensed under the terms of the GNU Lesser General Public License:
 * 		http://www.opensource.org/licenses/lgpl-license.php
 * 
 * For further information visit:
 * 		http://www.timkoop.com/niceditor
 * 
 * File Name: nictools.js
 * 	Utility functions.
 * 
 * Version:  2.0 RC2a
 * Modified: 2005-02-06 15:37:05 (nicified by Tim)
 *           2004-11-23 20:35:40
 * 
 * File Authors:
 * 		Tim Koop (nice at timkoop dot com)
 * 		Frederico Caldeira Knabben (fredck@fckeditor.net)
 */

var NICTools = new Object() ;

//**
// NICTools.GetLinkedFieldValue: Gets the value of the hidden INPUT element
// that is associated to the editor. This element has its ID set to the 
// editor's instance name so the user reffers to the instance name when getting
// the posted data.
NICTools.GetLinkedFieldValue = function()
{
	return NIC.LinkedField.value ;
}

//**
// NICTools.SetLinkedFieldValue: Sets the value of the hidden INPUT element
// that is associated to the editor. This element has its ID set to the 
// editor's instance name so the user reffers to the instance name when getting
// the posted data.
NICTools.SetLinkedFieldValue = function( value )
{
	NIC.LinkedField.value = value ;
}

//**
// NICTools.AttachToLinkedFieldFormSubmit: attaches a function call to the 
// submit event of the linked field form. This function us generally used to
// update the linked field value before submitting the form.
NICTools.AttachToLinkedFieldFormSubmit = function( functionPointer )
{
	// Gets the linked field form
	var oForm = NIC.LinkedField.form ;
	
	// Return now if no form is available
	if (!oForm) return ;

	// Attaches the functionPointer call to the onsubmit event
	if ( NICBrowserInfo.IsIE )
		oForm.attachEvent( "onsubmit", functionPointer ) ;
	else
		oForm.addEventListener( 'submit', functionPointer, true ) ;
	
	//**
	// Attaches the functionPointer call to the submit method 
	// This is done because IE doesn't fire onsubmit when the submit method is called
	// BEGIN --
	
	// Creates a Array in the form object that will hold all Attached function call
	// (in the case there are more than one editor in the same page)
	if (! oForm.updateNICEditor) oForm.updateNICEditor = new Array() ;
	
	// Adds the function pointer to the array of functions to call when "submit" is called
	oForm.updateNICEditor[oForm.updateNICEditor.length] = functionPointer ;

	// Switches the original submit method with a new one that first call all functions
	// on the above array and the call the original submit
	if (! oForm.originalSubmit && ! oForm.submit.tagName )
	{
		// Creates a copy of the original submit
		oForm.originalSubmit = oForm.submit ;
		
		// Creates our replacement for the submit
		oForm.submit = function()
		{
			if (this.updateNICEditor)
			{
				// Calls all functions in the functions array
				for (var i = 0 ; i < this.updateNICEditor.length ; i++)
					this.updateNICEditor[i]() ;
			}
			// Calls the original "submit"
			this.originalSubmit() ;
		}
	}
	// END --
}

//**
// NICTools.AddSelectOption: Adds a option to a SELECT element.
NICTools.AddSelectOption = function( targetDocument, selectElement, optionText, optionValue )
{
	var oOption = targetDocument.createElement("OPTION") ;

	oOption.text	= optionText ;
	oOption.value	= optionValue ;	

	selectElement.options.add(oOption) ;

	return oOption ;
}

NICTools.RemoveAllSelectOptions = function( selectElement )
{
	for ( var i = selectElement.options.length - 1 ; i >= 0 ; i-- )
	{
		selectElement.options.remove(i) ;
	}
}

NICTools.SelectNoCase = function( selectElement, value, defaultValue )
{
	var sNoCaseValue = value.toString().toLowerCase() ;
	
	for ( var i = 0 ; i < selectElement.options.length ; i++ )
	{
		if ( sNoCaseValue == selectElement.options[i].value.toLowerCase() )
		{
			selectElement.selectedIndex = i ;
			return ;
		}
	}
	
	if ( defaultValue != null ) NICTools.SelectNoCase( selectElement, defaultValue ) ;
}

NICTools.HTMLEncode = function( text )
{
	text = text.replace( /&/g, "&amp;" ) ;
	text = text.replace( /"/g, "&quot;" ) ;
	text = text.replace( /</g, "&lt;" ) ;
	text = text.replace( />/g, "&gt;" ) ;
	text = text.replace( /'/g, "&#39;" ) ;

	return text ;
}

//**
// NICTools.GetResultingArray: Gets a array from a string (where the elements 
// are separated by a character), a fuction (that returns a array) or a array.
NICTools.GetResultingArray = function( arraySource, separator )
{
	switch ( typeof( arraySource ) )
	{
		case "string" :
			return arraySource.split( separator ) ;
		case "function" :
			return separator() ;
		default :
			if ( isArray( arraySource ) ) return arraySource ;
			else return new Array() ;
	}
}

NICTools.GetElementPosition = function( el )
{
	// Initializes the Coordinates object that will be returned by the function.
	var c = { X:0, Y:0 } ;
	
	// Loop throw the offset chain.
	while ( el )
	{
		c.X += el.offsetLeft ;
		c.Y += el.offsetTop ;
		
		el = el.offsetParent ;
	}
	
	// Return the Coordinates object
	return c ;
}

NICTools.GetElementAscensor = function( element, ascensorTagName )
{
	var e = element.parentNode ;

	while ( e )
	{
		if ( e.nodeName == ascensorTagName )
			return e ;

		e = e.parentNode ;
	}
}

